#Open Autism Software. http://www.openautismsoftware.org
#This software is provided under the GNU General Public License, version 3
#See the license at http://www.gnu.org/licenses/gpl.html
#Authors: Thomas Hansen

from pymt import *
from OpenGL.GL import *

music_css = """
.label {
   font-size:32;
   }

.pause_button{
    font-size:0;
    draw-text-shadow:0;
    draw-background: 0;
    draw-alpha-background: 0;
    draw-border: 0;
    bg-image: "icons/pause.png";
    bg-image-down: "icons/play.png";
}

.record_button{
    font-size:0;
    draw-text-shadow:0;
    draw-background: 0;
    draw-alpha-background: 0;
    draw-border: 0;
    bg-image: "icons/record-grey.png";
    bg-image-down: "icons/record.png";
}
"""
css_add_sheet(music_css)


contolls_xml = """

<MTBoxLayout size_hint='(1, None)' spacing='10'>
    <MTButton id='"record"' cls='"record_button"' />
    <MTToggleButton id='"pause"' cls='"pause_button"' />
    <MTWidget width='50'/>
    <MTBoxLayout size_hint='(1, 1)'>
        <MTLabel label='"Speed:"' size_hint='(None,1)' anchor_y='"center"' cls='"label"'/>
        <MTSlider  orientation='"horizontal"' size_hint='(1,1)' min='30' max='300' value='120'
            id='"speed"' cls='"speed_slider"'/>
    </MTBoxLayout>
</MTBoxLayout>

"""

class MusicApp(MTBoxLayout):
    def __init__(self, **kwargs):
        kwargs['orientation'] = 'vertical'
        super(MusicApp, self).__init__(**kwargs)

        self.controlls = XMLWidget(xml=contolls_xml)
        self.add_widget(self.controlls.root)
        self.controlls.autoconnect(self)

        self.music_grid = MusicGrid(matrix_size=(16,16), size_hint=(1,1))
        self.add_widget(self.music_grid)

    def on_pause_press(self, *args):
        self.music_grid.playing = self.controlls.getById('pause').state == 'normal'

    def on_speed_value_change(self, value):
        self.music_grid.bpm = value

    def on_record_press(self, *args):
        self.music_grid.reset()
        getWindow().dispatch_event('on_keyboard',293,None, None)


class Kalimba(object):
    def __init__(self):
        self.sound = []
        for i in range(16):
            soundfile = "instruments/kalimba/kalimba%s.ogg" % (i+1)
            self.sound.append(SoundLoader.load(soundfile))

    def play(self, note):
        self.sound[note].stop()
        self.sound[note].play()


class MusicGrid(MTButtonMatrix):
    def __init__(self, **kwargs):
        super(MusicGrid, self).__init__(**kwargs)
        self.instrument = Kalimba()
        self.playing = True
        self.bpm = 120
        self.play_pos = 0
        self.t = 0

    def advance(self):
        self.play_pos = (self.play_pos + 1) % self._matrix_size[0]
        for i in range(self._matrix_size[1]):
            if self.matrix[self.play_pos][i] == 1:
                self.instrument.play(i)

    def on_update(self):
        if self.playing:
            self.t += getFrameDt()
            if self.t > 60.0/self.bpm:
                self.advance()
                self.t = 0

    def on_touch_move(self, touch):
        if self.collide_point(touch.x, touch.y):
            i,j = self.collide_point(touch.x, touch.y)
            for r in range(len(self.matrix[1])):
                for c in range(len(self.matrix[1])):
                    if self.matrix[r][c] != 1:
                        self.matrix[r][c] = 0
            if self.matrix[i][j] == 0:
                self.matrix[i][j] = 0.5


    def on_touch_down(self, touch):
        if self.collide_point(touch.x, touch.y):
            i,j = self.collide_point(touch.x, touch.y)
            if self.matrix[i][j] == 0:
                self.matrix[i][j] = 0.5


    def on_touch_up(self, touch):
        if self.collide_point(touch.x, touch.y):
            i,j = self.collide_point(touch.x, touch.y)
            for r in range(len(self.matrix[1])):
                for c in range(len(self.matrix[1])):
                    if self.matrix[r][c] != 1:
                        self.matrix[r][c] = 0
            if self.matrix[i][j] == 1:
                self.matrix[i][j] = 0
            else:
                self.matrix[i][j] = 1
            self.dispatch_event('on_value_change', self.matrix)
            self.dispatch_event('on_press', (i,j, self.matrix[i][j]))
            self.last_tile = (i,j)

    def draw(self):
        w,h = self.width/self._matrix_size[0], self.height/self._matrix_size[1]
        glPushMatrix()
        glTranslate(self.x, self.y, 0)
        for col in range (self._matrix_size[1]):
            note_color = (1,0.5,0,0.8)
            glPushMatrix()
            if col == self.play_pos:
                note_color = (1,0.2,0.2, 1.0)
                padding = 0
                if self.playing:
                    set_color(0,1,0,0.8)
                else:
                    set_color(0.5,0.5,0.5)
                drawRectangle(size=(w, self.height))
            for row in range (self._matrix_size[1]):
                if self.matrix[col][row] == 0:
                    set_color(0.7,0.7,0.7,0.5)
                if self.matrix[col][row] == 0.5:
                    set_color(0.5,0.5,0.5,0.5)
                if self.matrix[col][row] == 1:
                    set_color(*note_color)
                drawRectangle(pos=(4,4),size=(w-8,h-8))
                glTranslate(0, h, 0)
            glPopMatrix()

            glTranslate(w, 0, 0)
        glPopMatrix()


runTouchApp(MusicApp())
